﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Logica.Kbn.PostSharpAspect.Extension;
using PostSharp.Laos;

namespace Logica.Kbn.PostSharpAspect
{
    /// <summary>
    /// logging aspect for console logging
    /// </summary>
    [Serializable()]
    public class LoggingAspect : OnMethodBoundaryAspect
    {
        #region properties & constructors

        /// <summary>
        /// enables console logging
        /// </summary>
        public bool UseDebug { get; set; }

        /// <summary>
        /// enables trace logging
        /// </summary>
        public bool UseTrace { get; set; }

        /// <summary>
        /// enables console logging
        /// </summary>
        public bool UseConsole { get; set; }

        /// <summary>
        /// Creates a new logging aspect instance
        /// UseDebug = TRUE
        /// UseTrace = TRUE
        /// UseConsole = TRUE
        /// </summary>
        public LoggingAspect()
        {
            this.UseDebug = true;
            this.UseTrace = true;
            this.UseConsole = true;
        }

        #endregion

        #region helper methods

        private void Log(string message)
        {
            this.Debug(message);
            this.Trace(message);
            this.Console(message);
        }

        private void Debug(string message)
        {
            if (!this.UseDebug) return;

            System.Diagnostics.Debug.WriteLine(message);
        }

        private void Trace(string message)
        {
            if (!this.UseTrace) return;

            System.Diagnostics.Trace.WriteLine(message);
        }

        private void Console(string message)
        {
            if (!this.UseConsole) return;

            System.Console.WriteLine(message);
        }

        #endregion

        #region overridden methods

        public override void OnEntry(MethodExecutionEventArgs eventArgs)
        {
            string instanceName = eventArgs.Instance == null ? "STATISCHES OBJEKT" : eventArgs.Instance.ToString();
            string message = string.Format("Entering {0} on object {1}", eventArgs.Method, instanceName);

            this.Log(message);
        }

        public override void OnExit(MethodExecutionEventArgs eventArgs)
        {
            string instanceName = eventArgs.Instance == null ? "STATISCHES OBJEKT" : eventArgs.Instance.ToString();
            string message = string.Format("Exiting {0} on object {1}", eventArgs.Method, instanceName);

            this.Log(message);
        }

        public override void OnException(MethodExecutionEventArgs eventArgs)
        {
            string instanceName = eventArgs.Instance == null ? "STATISCHES OBJEKT" : eventArgs.Instance.ToString();
            string message = string.Format("Exception {0} on object {1}", eventArgs.Method, instanceName) + Environment.NewLine;
            message = message.Space(4).Append(eventArgs.Exception.Message);

            this.Log(message);

            eventArgs.FlowBehavior = FlowBehavior.Continue;
        }

        #endregion

    }
}
